using System;
using System.Collections;
using OVRSimpleJSON;
using TMPro;
using UnityEngine;
using UnityEngine.Networking;

/// <summary>
///     This script communicates with our score web server.
///     https://gitlab.beuth-hochschule.de/vrar-ws21-gruppe-k/score-web-server
///     For the script to work, the appKey must be available via the WebConfig script.
///     This is described in the readme.
///     The WebConfig does not end up in the remote repo!
/// </summary>
public class Web : MonoBehaviour
{
    /// <summary>
    ///     Our server can be reached at this address.
    /// </summary>
    [SerializeField] private string host = "http://127.0.0.1:5000";

    /// <summary>
    ///     What is the name of this game.
    /// </summary>
    [SerializeField] private string gameTitle;

    /// <summary>
    ///     This is the TMP_Text object in the menu, which displays the name of the game.
    /// </summary>
    [Header("Text")] [SerializeField] private TMP_Text gameTitleText;

    /// <summary>
    ///     This is the TMP_Text object in the menu, which displays the autogenerated name of the player.
    /// </summary>
    [SerializeField] private TMP_Text playerNameText;

    /// <summary>
    ///     This is the TMP_Text object in the menu, which displays the scores of the 10 best players of this game.
    /// </summary>
    [SerializeField] private TMP_Text scoreBoardText;
    
    [SerializeField] private int iMaximumNumberOfScoresToDisplay = 3;

    /// <summary>
    ///     Player ID (public).
    ///     Is needed to query and update the score.
    /// </summary>
    private int _id;

    /// <summary>
    ///     Player key (secret).
    ///     Is needed to update the score.
    /// </summary>
    private string _key;

    /// <summary>
    ///     The autogenerated name of the player.
    /// </summary>
    private string _name;

    /// <summary>
    ///     The score of the player.
    /// </summary>
    private int _score;


    // Start is called before the first frame update
    private void Start()
    {
        gameTitleText.text = gameTitle;
        playerNameText.text = "";
        scoreBoardText.text = "Connect to Wifi to enable the Scoreboard!";

        var savedKey = PlayerPrefs.GetString("key");
        Debug.Log(savedKey);
        if (string.IsNullOrEmpty(savedKey))
        {
            var newKey = Guid.NewGuid().ToString();
            PlayerPrefs.SetString("key", newKey);
            savedKey = newKey;
        }

        _key = savedKey;
        Init();
    }

    // Update is called once per frame
    private void Update()
    {
        /*
        if (Input.GetKeyDown(KeyCode.Space))
        {
            _score += 37293;
            SaveAndGetList();
        }
        */
    }

    private void UpdateGUI()
    {
        gameTitleText.text = gameTitle;
        playerNameText.text = _name + "#" + _id + ": " + PlayerPrefs.GetInt("Score");
    }

    /// <summary>
    ///     Register the player and get the name, ID and current score.
    /// </summary>
    private void Init()
    {
        var form = new WWWForm();
        form.AddField("app_key", WebConfig.AppKey);
        form.AddField("key", _key);
        StartCoroutine(Post(host + "/api/init/", form));
    }

    /// <summary>
    ///     Update the score on the server and get a message back, matching the performance of the player.
    /// </summary>
    private void UpdateScore()
    {
        /*
        var scoreManager = FindObjectOfType<Score>();
        if (!scoreManager) return;
        _score = scoreManager.GetCurrentScore();
        */
        _score = PlayerPrefs.GetInt("Score");

        var form = new WWWForm();
        form.AddField("app_key", WebConfig.AppKey);
        form.AddField("key", _key);
        form.AddField("id", _id);
        form.AddField("score", _score);
        StartCoroutine(Post(host + "/api/score/", form));
    }

    /// <summary>
    ///     Receive the 10 best scores.
    /// </summary>
    private void GetList()
    {
        StartCoroutine(Get(host + "/api/list/" + WebConfig.AppKey + "/" + _id + "/"));
    }

    /// <summary>
    ///     Update the score first and then get back the list with the best scores.
    /// </summary>
    public void SaveAndGetList()
    {
        UpdateScore();
        GetList();
        UpdateGUI();
    }

    /// <summary>
    ///     The post request for Init() or UpdateScore().
    /// </summary>
    /// <param name="url"></param>
    /// <param name="form"></param>
    /// <returns></returns>
    private IEnumerator Post(string url, WWWForm form)
    {
        using (var www = UnityWebRequest.Post(url, form))
        {
            yield return www.SendWebRequest();

            if (www.result != UnityWebRequest.Result.Success)
            {
                Debug.Log(www.error);
            }
            else
            {
                var responseString = www.downloadHandler.text;
                var dict = JSONNode.Parse(responseString);
                foreach (var keysValuePair in dict)
                {
                    var key = keysValuePair.Key;
                    var value = keysValuePair.Value;

                    Debug.Log(key + ": " + value);

                    switch (key)
                    {
                        case "id":
                            _id = value.AsInt;
                            break;
                        case "name":
                            _name = value.ToString().Replace("\"", "");
                            break;
                        case "score":
                            _score = value.AsInt;
                            break;
                        case "message":
                            Debug.Log(value.ToString());
                            break;
                    }

                    //GetList();
                    SaveAndGetList();
                }
            }
        }
    }

    /// <summary>
    ///     The get-request for querying the score list.
    /// </summary>
    /// <param name="uri"></param>
    /// <returns></returns>
    private IEnumerator Get(string uri)
    {
        using (var webRequest = UnityWebRequest.Get(uri))
        {
            // Request and wait for the desired page.
            yield return webRequest.SendWebRequest();

            var pages = uri.Split('/');
            var page = pages.Length - 1;

            switch (webRequest.result)
            {
                case UnityWebRequest.Result.ConnectionError:
                case UnityWebRequest.Result.DataProcessingError:
                    Debug.LogError(pages[page] + ": Error: " + webRequest.error);
                    break;
                case UnityWebRequest.Result.ProtocolError:
                    Debug.LogError(pages[page] + ": HTTP Error: " + webRequest.error);
                    break;
                case UnityWebRequest.Result.Success:
                    //Debug.Log(pages[page] + ":\nReceived: " + webRequest.downloadHandler.text);
                    var responseString = webRequest.downloadHandler.text;
                    var dict = JSONNode.Parse(responseString);

                    var newScoreBoard = "";
                    var rank = 1;
                    
                    foreach (var score in dict["scores"])
                    {
                        var value = score.Value;
                        var valueParts = value.ToString().Split(';');
                        var playerName = valueParts[0].Replace("\"", "");
                        var playerScore = valueParts[1];

                        newScoreBoard += rank + ": " + playerName + " | Score: " + playerScore;

                        Debug.Log(playerName);
                        if (playerName == _name + "#" + _id)
                            newScoreBoard += " (you)";

                        newScoreBoard += "\n";

                        rank++;

                        if (rank >= iMaximumNumberOfScoresToDisplay) break;
                    }

                    scoreBoardText.text = newScoreBoard.Replace("\"", "");
                    UpdateGUI();
                    break;
            }
        }
    }
}